//+------------------------------------------------------------------+
//|                              Jim Sloman's natural moving average |
//|                                                      ocn nma.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  Orange
#property indicator_color3  Orange
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

//
//
//
//
//

enum enPrices
{
   pr_close,   // Close
   pr_open,    // Open
   pr_high,    // High
   pr_low,     // Low
   pr_median,  // Median
   pr_typical, // Typical
   pr_weighted // Weighted
};

extern string   TimeFrame     = "Current time frame";
extern int      NMAPeriod     = 40;
extern enPrices NMAPrice      = pr_close;
extern int      TEMAPeriod    = 15;
extern int      SmoothPeriod  = 8;
extern bool     SmoothUsingSSM = true;
extern bool     Interpolate   = true;

//
//
//
//
//

double nma[];
double nmas[];
double nmasda[];
double nmasdb[];
double slope[];
double coef1;
double coef2;
double coef3;

string indicatorFileName;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define Pi 3.14159265358979323846264338327950288
int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,nmas);
   SetIndexBuffer(1,nmasda);
   SetIndexBuffer(2,nmasdb);
   SetIndexBuffer(3,nma);
   SetIndexBuffer(4,slope);

      //
      //
      //
      //
      //
         
      TEMAPeriod = MathMax(TEMAPeriod,1);
      NMAPeriod  = MathMax(NMAPeriod ,1);
           SmoothPeriod  = MathMax(SmoothPeriod,1);
                double a = MathExp(-1.414*Pi/SmoothPeriod);
                double b = 2*a*MathCos(1.414*Pi/SmoothPeriod);

                  coef2 = b;
                  coef3 = -a*a;
                  coef1 = 1-coef2-coef3;

                  timeFrame         = stringToTimeFrame(TimeFrame);   
                  indicatorFileName = WindowExpertName();
                  returnBars        = (TimeFrame == "returnBars");

   IndicatorShortName (timeFrameToString(timeFrame)+" NMA ("+NMAPeriod+")");
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double tBuffer[][2];
#define iPrc 0
#define iMom 1

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { nmas[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (slope[limit]==-1) CleanPoint(limit,nmasda,nmasdb);
   if (timeFrame == Period())
   {
      if (ArrayRange(tBuffer,0) != Bars) ArrayResize(tBuffer,Bars);
      for(int i=limit, r=Bars-i-1; i >= 0; i--,r++)
      {
         double rawPrice = iMA(NULL,0,1,0,MODE_SMA,(int)NMAPrice,i);
      
         tBuffer[r][iPrc] = iTema(rawPrice,TEMAPeriod,i);
         tBuffer[r][iMom] = tBuffer[r][iPrc]-tBuffer[r-1][iPrc];
   
         //
         //
         //
         //
         //
                      
            double momRatio = 0.00;
            double sumMomen = 0.00;
            double ratio    = 0.00;
      
            for (int k = 0; k<NMAPeriod; k++)
            {
               sumMomen += MathAbs(tBuffer[r-k][iMom]);
               momRatio +=         tBuffer[r-k][iMom]*(MathSqrt(k+1)-MathSqrt(k));
            }
            if (sumMomen != 0) ratio = MathAbs(momRatio)/sumMomen;
      
         //
         //
         //
         //
         //

         nmasda[i] = EMPTY_VALUE;
         nmasdb[i] = EMPTY_VALUE;
         nma[i] = nma[i+1]+ratio*(rawPrice-nma[i+1]);
         if (i >= Bars-2)
               nmas[i] = nma[i];
         else  
            if (SmoothUsingSSM)
                  nmas[i] = iSsm(nma[i],SmoothPeriod,i);
            else  nmas[i] = coef1*nma[i]+coef2*nmas[i+1]+coef3*nmas[i+2];         
         slope[i] = slope[i+1];
         if (nmas[i]>nmas[i+1]) slope[i] =  1;
         if (nmas[i]<nmas[i+1]) slope[i] = -1;
               if (slope[i] == -1) PlotPoint(i,nmasda,nmasdb,nmas);
      }
      return(0);
   }      
   
   //
   //
   //
   //
   //

   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period())); 
   if (slope[limit]==-1) CleanPoint(limit,nmasda,nmasdb);
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         nmas[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",NMAPeriod,NMAPrice,TEMAPeriod,SmoothPeriod,SmoothUsingSSM,0,y);
         slope[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",NMAPeriod,NMAPrice,TEMAPeriod,SmoothPeriod,SmoothUsingSSM,4,y);
         nmasda[i] = EMPTY_VALUE;
         nmasdb[i] = EMPTY_VALUE;

         //
         //
         //
         //
         //

         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
            datetime time = iTime(NULL,timeFrame,y);
               for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
               for(    k = 1; k<n; k++)
                  nmas[i+k] = nmas[i] + (nmas[i+n] - nmas[i])*k/n;
   }
   for(i=limit; i>=0; i--) if (slope[i] == -1) PlotPoint(i,nmasda,nmasdb,nmas);
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double workTema[][3];
#define _tema1 0
#define _tema2 1
#define _tema3 2

double iTema(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workTema,0)!= Bars) ArrayResize(workTema,Bars); instanceNo*=3; r = Bars-r-1;

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+period);
          workTema[r][_tema1+instanceNo] = workTema[r-1][_tema1+instanceNo]+alpha*(price                         -workTema[r-1][_tema1+instanceNo]);
          workTema[r][_tema2+instanceNo] = workTema[r-1][_tema2+instanceNo]+alpha*(workTema[r][_tema1+instanceNo]-workTema[r-1][_tema2+instanceNo]);
          workTema[r][_tema3+instanceNo] = workTema[r-1][_tema3+instanceNo]+alpha*(workTema[r][_tema2+instanceNo]-workTema[r-1][_tema3+instanceNo]);
   return(workTema[r][_tema3+instanceNo]+3.0*(workTema[r][_tema1+instanceNo]-workTema[r][_tema2+instanceNo]));
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

int stringToTimeFrame(string tfs)
{
   StringToUpper(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
         if (first[i+2] == EMPTY_VALUE) 
               { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
         else  { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else        { first[i]  = from[i];                          second[i] = EMPTY_VALUE; }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

double workSsm[][2];
#define _tprice  0
#define _ssm    1

double workSsmCoeffs[][4];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3

//
//
//
//
//

double iSsm(double price, double period, int i, int instanceNo=0)
{
   if (ArrayRange(workSsm,0) !=Bars)                 ArrayResize(workSsm,Bars);
   if (ArrayRange(workSsmCoeffs,0) < (instanceNo+1)) ArrayResize(workSsmCoeffs,instanceNo+1);
   if (workSsmCoeffs[instanceNo][_period] != period)
   {
      workSsmCoeffs[instanceNo][_period] = period;
      double a1 = MathExp(-1.414*Pi/period);
      double b1 = 2.0*a1*MathCos(1.414*Pi/period);
         workSsmCoeffs[instanceNo][_c2] = b1;
         workSsmCoeffs[instanceNo][_c3] = -a1*a1;
         workSsmCoeffs[instanceNo][_c1] = 1.0 - workSsmCoeffs[instanceNo][_c2] - workSsmCoeffs[instanceNo][_c3];
   }

   //
   //
   //
   //
   //

      int s = instanceNo*2;   
      i = Bars-i-1;
          workSsm[i][s+_tprice] = price;
          if (i<2)
               workSsm[i][s+_ssm] = price;
          else workSsm[i][s+_ssm]  = workSsmCoeffs[instanceNo][_c1]*(workSsm[i][s+_tprice]+workSsm[i-1][s+_tprice])/2.0 + 
                                     workSsmCoeffs[instanceNo][_c2]*workSsm[i-1][s+_ssm]                                + 
                                     workSsmCoeffs[instanceNo][_c3]*workSsm[i-2][s+_ssm]; 
   return(workSsm[i][s+_ssm]);
}